@@ -1,7 +1,9 @@
 package Gtk2::Ex::DbLinker::AbForm;
+use Class::InsideOut qw(public private register id);
 use Gtk2::Ex::DbLinker::DbTools;
+use Scalar::Util qw(weaken);
+use Carp 'croak';
 our $VERSION = $Gtk2::Ex::DbLinker::DbTools::VERSION;
-
 =head1 NAME
 
 Gtk2::Ex::DbLinker::AbForm - Common methods for Gtk2::Ex::DbLinker::Form and Wx::Perl::DbLinker::Wxform
@@ -18,14 +20,88 @@ use Data::Dumper;
 use DateTime::Format::Strptime;
 use Carp qw(confess);
 
+private data_manager => my %dman;
+public child_class => my %child_class;
+private log => my %log;
+private event => my %events;
+private states => my %states;
+private widgets => my %widgets;
+
+my @arg_names;
+
 sub new {
 
     my $class = shift;
+    my $self = \( my $scalar );
+    bless $self, $class;
+     register $self;
+ weaken $self;
+    my $id = id $self;
     my @arg   = @_;
     my $def   = {};
-    my $self  = { ( %$def, @arg ) };
-    $self->{log} = Log::Log4perl->get_logger(__PACKAGE__);
-    bless $self, $class;
+    my $arg_value_ref  = { ( %$def, @arg ) };
+
+    my $arg_holder_ref = { 
+        childclass=> \%child_class, 
+        data_manager=> \%dman,
+        datawidgets => \$widgets{ $id}->{cols},
+        datawidgets_ro => \$widgets{ $id}->{datawidgets_ro},
+        builder => \$widgets{ $id }->{builder}, 
+        on_current => \$events{ $id }->{on_current},
+        date_formatters => \$widgets{ $id }->{date_formatters},
+        time_zone => \$widgets{ $id }->{time_zone},
+        locale => \$widgets{ $id }->{locale},
+        rec_spinner_callback => \$events{ $id }->{rec_spinner_callback},
+        rec_spinner_insert_callback => \$events{ $id}->{rec_spinner_insert_callback},
+
+    } ;
+
+     @arg_names = keys %{$arg_holder_ref};
+
+    for my $name (@arg_names){
+        next unless defined ($arg_value_ref->{$name});
+
+        if (ref $arg_holder_ref->{$name} eq "HASH"){
+            $arg_holder_ref->{$name}->{ $id } = $arg_value_ref->{$name};
+        } 
+        #elsif (ref $arg_holder_ref->{$name} eq "ARRAY") {
+
+        #} 
+        else {
+           ${$arg_holder_ref->{$name} } =  $arg_value_ref->{$name};
+        
+        }
+    
+    }
+    $log{ $id } = Log::Log4perl->get_logger(__PACKAGE__);
+        my @dates;
+
+    #$self->{subform} = [];
+
+    #my %formatters_db;
+    #my %formatters_f;
+    # $self->{dates_formatted} = \(keys %{$self->{date_formatters}});
+    if ( !defined $widgets{ $id }->{cols} ) {
+        my @col = $dman{ $id }->get_field_names;
+        $widgets{ $id }->{cols} = \@col;
+    }
+    foreach my $v ( keys %{ $widgets{ $id }->{date_formatters} } ) {
+        $log{ $id }->debug( "** " . $v . " **" );
+        push @dates, $v;
+    }
+    $widgets{ $id }->{dates_formatted} = \@dates;
+    my %hdates = map { $_ => 1 } @dates;
+    $widgets{ $id }->{hdates_formatted} = \%hdates;
+    $widgets{ $id }->{dates_formatters} = {};
+    $states{ $id }->{inserting}        = 0;
+    $widgets{ $id }->{pos2del}          = [];
+
+    #bless $self, $class;
+   return $self;
+}
+
+sub _super_args_needed {
+        return @arg_names;
 }
 
 =head2 C<set_data_manager( $dman ) >
@@ -36,7 +112,7 @@ Replaces the current data manager with the one receives. The columns should not
 
 sub set_data_manager {
     my ( $self, $dman ) = @_;
-    $self->{dman} = $dman;
+    $dman{ id $self } = $dman;
 }
 
 =item C<add_childform( $childform )>
@@ -47,52 +123,124 @@ You may add any dependant form or datasheet object with this call if you want th
 
 sub add_childform {
     my ( $self, $sf ) = @_;
-    $self->{log}->warn(
+    my $id = id $self;
+    $log{ $id }->warn(
         "add_childform : do not set auto_apply to 0 if you call this method")
-      unless ( $self->{auto_apply} );
+      unless ( $states{ $id }->{auto_apply} );
 
 #carp("add_childform : do not set auto_apply to 0 if you call this method")  unless ($self->{auto_apply});
-    push @{ $self->{subform} }, $sf;
+    push @{ $widgets{ $id }->{subform} }, $sf;
+    weaken @{ $widgets{ $id }->{subform} }[-1];
+
 
 }
 
 sub _init {
     my $self = shift;
-
-    if ( defined $self->{datawidgets_ro} ) {
+    my $id = id $self;
+    if ( defined $widgets{ $id }->{datawidgets_ro} ) {
         my %seen;
         %seen = map { $_ => $seen{$_}++ }
-          ( @{ $self->{cols} }, @{ $self->{datawidgets_ro} } );
+          ( @{ $widgets{ $id }->{cols} }, @{ $widgets{ $id }->{datawidgets_ro} } );
         my @fields_to_save = grep { $seen{$_} < 1 } keys %seen;
 
-        #$self->{log}->debug("cols: " . join(" ", @{$self->{cols}}));
-        $self->{log}->debug( "cols to saved: " . join( " ", @fields_to_save ) );
-        $self->{col2save} = \@fields_to_save;
+        #$log{ $id }->debug("cols: " . join(" ", @{$self->{cols}}));
+        $log{ $id }->debug( "cols to saved: " . join( " ", @fields_to_save ) );
+        $widgets{ $id }->{col2save} = \@fields_to_save;
 
     } else {
-        $self->{col2save} = $self->{cols};
+        $widgets{ $id }->{col2save} = $widgets{ $id }->{cols};
 
     }
 
 }
 
+sub _painting {
+    my $self = shift;
+    my $idob = id $self;
+    $states{ $idob}->{painting} = $_[0] if (defined $_[0]);
+    return  $states{ $idob}->{painting};
+
+}
+
+sub _changed {
+    my $self = shift;
+    my $idob = id $self;
+    $states{ $idob}->{changed} = $_[0] if (defined $_[0]);
+    return  $states{ $idob }->{changed};
+
+}
+
+sub _builder {
+    my $self = shift;
+    my $idob = id $self;
+    $widgets{ $idob}->{builder} = $_[0] if (defined $_[0]);
+    return  $widgets{ $idob}->{builder};
+
+}
+#this datawidgets hash has nothing to do with the argument datawidgets 
+#used in new ... it just shows my lake of imagination
+sub _datawidgets {
+    my $self = shift;
+    my $idob = id $self;
+    $widgets{ $idob}->{datawidgets}->{ $_[0]} = $_[1] if (defined $_[1]);
+    return  $widgets{ $idob}->{datawidgets}->{ $_[0] } if (defined $_[0]); 
+    return  $widgets{ $idob}->{datawidgets};
+
+}
+sub _datawidgetsName {
+    my $self = shift;
+    my $idob = id $self;
+    $widgets{ $idob}->{datawidgetsName}->{ $_[0] } = $_[1] if (defined $_[1]);
+    return  $widgets{ $idob}->{datawidgetsName}->{ $_[0] } if (defined $_[0]); 
+    return  $widgets{ $idob}->{datawidgetsName};
+
+}
+
+sub _cols {
+    my $self = shift;
+  confess __PACKAGE__ . "_cols is readonly" if (defined $_[0]);
+    return  $widgets{ id $self}->{cols};
+}
+
+sub _dman {
+     my $self = shift;
+     confess __PACKAGE__ . "_dman is readonly" if (defined $_[0]);
+    return  $dman{ id $self};
+
+}
+
+sub _pos {
+     my $self = shift;
+     confess __PACKAGE__ . "_pos is readonly" if (defined $_[0]);
+    return  $states{ id $self}->{pos};
+
+}
+
+sub _auto_apply {
+     my $self = shift;
+      confess __PACKAGE__ . "_auto_apply is readonly" if (defined $_[0]);
+    return  $states{ id $self }->{auto_apply};
+}
+
 sub _display_data {
     my ( $self, $pos ) = @_;
-    $self->{log}->debug( "display_data for row at pos " . $pos );
+     my $idob = id $self;
+    $log{ $idob }->debug( "display_data for row at pos " . $pos );
 
-    my $dman = $self->{dman};
+    my $dman = $dman{ $idob };
 
-    $self->{pos} = $pos;
+    $states{ $idob }->{pos} = $pos;
 
     $dman->set_row_pos($pos) unless ( $pos < 0 );
 
-    $self->{painting} = 1;
+    $states{ $idob}->{painting} = 1;
 
     #foreach my $id (keys %{$self->{datawidgets}}){
-    foreach my $id ( @{ $self->{cols} } ) {
+    foreach my $id ( @{ $widgets{ $idob }->{cols} } ) {
 
-        my $w    = $self->{datawidgets}->{$id};
-        my $name = $self->{datawidgetsName}->{$id};
+        my $w    = $widgets{ $idob }->{datawidgets}->{$id};
+        my $name = $widgets{ $idob }->{datawidgetsName}->{$id};
 
 #die("no name found $id") unless($name);
 #if $name is not defined means that $id is in a field array but with no corresponding
@@ -109,14 +257,15 @@ sub _display_data {
             #$x = $row->$id() if ($row);
             $x = $dman->get_field($id);
             my $ref = ref $x;
-            $self->{log}->debug( "ref: " . $ref ) if ($ref);
+            $log{ $idob }->debug( "ref: " . $ref ) if ($ref);
             if ( $ref && $ref eq "ARRAY" ) {
 
                 # my @set = $row->$id();
                 #my @set = $dman->get_field($id);
                 my @set = @$x;
                 $x = join( ',', @set );
-                $self->{log}->debug( "id: "
+                #confess("dman undef") unless ($dman{ $idob });
+                $log{ $idob }->debug( "id: "
                       . $id
                       . " gtkname : "
                       . $name
@@ -125,7 +274,7 @@ sub _display_data {
                       . " value: "
                       . ( $x ? $x : "" )
                       . " type : "
-                      . $self->{dman}->get_field_type($id) );
+                      . $dman{ $idob }->get_field_type($id) );
 
             }
 
@@ -133,25 +282,25 @@ sub _display_data {
 
         # $w->signal_handler_block()
 
-        if ( defined $self->{hdates_formatted}->{$id} ) {
+        if ( defined $widgets{ $idob }->{hdates_formatted}->{$id} ) {
 
             #$x = $self->_dateformatter($self->{date_formatters}->{$id}, $x);
             if ( defined $x ) {
 
             #my $ff = $self->{dates_formatters_f}->{$id};
             #my $fdb = $self->{dates_formatters_db}->{$id};
-            # $self->{log}->debug("display_data formatted received date: ". $x);
+            # $log{ $id }->debug("display_data formatted received date: ". $x);
                 $x = $self->_format_date( 0, $id, $x );
 
             }
         }
-        $self->{log}->debug( $name . " widget undef " ) unless ($w);
+        $log{ $idob }->debug( $name . " widget undef " ) unless ($w);
 
-        my %setter = $self->{childclass}->_get_setter;
+        my %setter = $child_class{ $idob }->_get_setter;
         $setter{$name}( $self, $w, $x, $id ) if ( $name && $setter{$name} );
 
         if ( $name eq "Wx::ListCtrl" ) {
-            $self->{datawidgetsValue}->{$id} = $x;
+            $widgets{ $idob }->{datawidgetsValue}->{$id} = $x;
 
         }
     }    #foreach
@@ -161,14 +310,14 @@ sub _display_data {
 
 =for comment
     my $first = ($pos < 0 ? 0 : 1);
-   $self->_set_rs_range($first,  $self->{dman}->row_count);
+   $self->_set_rs_range($first,  $dman{ $id }->row_count);
    my $coderef = $self->{rec_spinner_callback};
    &$coderef($self);
 =cut
 
-    $self->{on_current}() if ( $self->{on_current} );
-    $self->{painting} = 0;
-    $self->{changed}  = 0;
+    $events{ $idob }->{on_current}() if ( $events{ $idob }->{on_current} );
+    $states{ $idob }->{painting} = 0;
+    $states{ $idob }->{changed}  = 0;
 
 }
 
@@ -184,34 +333,35 @@ Create an empty rows at position 0 in the record_count_label.
 
 sub insert {
     my $self = shift;
-    $self->{log}->debug("insert");
+    my $id = id $self;
+    $log{ $id }->debug("insert");
 
     # my $row = $self->{data}[0]->new;
-    $self->{inserting} = 1;
+    $states{ $id }->{inserting} = 1;
 
     #$self->{pos} = $self->{count} + 1;
     #afficher des champs vides
     #$self->_display_data(-1);
-    my $new_pos = $self->{dman}->row_count;
+    my $new_pos = $dman{ $id }->row_count;
     my $first = ( $new_pos > 0 ? 1 : 0 );
 
 #data_manager->new_row is called when apply is cliked / but defaults value are not displayed then
-    $self->{dman}->new_row;
-    $self->{log}->debug(
+    $dman{ $id }->new_row;
+    $log{ $id }->debug(
         "insert : row count is : ",
-        $self->{dman}->row_count,
+        $dman{ $id }->row_count,
         " new pos is : ", $new_pos
     );
 
 # SqlADM and DBIDM ->new_row does not change row_count (are the others DM similar ?)
     $self->_display_data($new_pos);
-    $self->{log}->debug( ref $self );
-    my $coderef = $self->{rec_spinner_insert_callback};
+    $log{ $id }->debug( ref $self );
+    my $coderef = $events{ $id }->{rec_spinner_insert_callback};
     &$coderef( $self, $new_pos );
 
 =for comment
 	   if ($self->{rec_spinner}){
-		  #	my $last = $self->{dman}->row_count;
+		  #	my $last = $dman{ $id }->row_count;
 		  #$self->{rec_spinner}->signal_handler_block( $self->{rs_value_changed_signal} );
 	         $self->{rec_spinner}->SetRange( $first, $new_pos+1 );
 		 #ne pas appler _rs_on_changed ici
@@ -232,12 +382,13 @@ Revert the row to the original state in displaying the values fetch from the dat
 
 sub undo {
     my $self = shift;
-    $self->{log}->debug("undo clicked");
-    $self->{changed}   = 0;
-    $self->{inserting} = 0;
-    $self->{pos2del}   = [];
-    $self->_display_data( $self->{pos} );
-    my $coderef = $self->{rec_spinner_callback};
+    my $id = id $self;
+    $log{ $id }->debug("undo clicked");
+    $states{ $id }->{changed}   = 0;
+    $states{ $id }->{inserting} = 0;
+    $widgets{ $id }->{pos2del}   = [];
+    $self->_display_data( $states{ $id }->{pos} );
+    my $coderef = $events{ $id }->{rec_spinner_callback};
     &$coderef($self);
 
 =for comment
@@ -263,11 +414,12 @@ Marks the current row to be deleted. The deletion itself will be done on apply.
 
 sub delete {
     my $self = shift;
-    $self->{log}->debug( "delete at " . $self->{dman}->get_row_pos );
+    my $id = id $self;
+    $log{ $id }->debug( "delete at " . $dman{ $id }->get_row_pos );
 
     #$self->next;
-    $self->{changed} = 1;
-    push @{ $self->{pos2del} }, $self->{dman}->get_row_pos;
+    $states{ $id }->{changed} = 1;
+    push @{ $widgets{ $id }->{pos2del} }, $dman{ $id }->get_row_pos;
     $self->_set_record_status_label;
 
 }
@@ -280,9 +432,10 @@ return true if the data exposed in the current row has been modified. If autoapl
 
 sub has_changed {
     my $self   = shift;
-    my $result = $self->{changed};
-    if ( $self->{auto_apply} ) {
-        foreach my $sf ( @{ $self->{subform} } ) {
+    my $id = id $self;
+    my $result = $states{ $id }->{changed};
+    if ( $states{ $id }->{auto_apply} ) {
+        foreach my $sf ( @{ $widgets{ $id }->{subform} } ) {
             if ( $sf->has_changed ) {
                 $result = 1;
                 last;
@@ -308,28 +461,29 @@ When inserting a new row, you can pass an array ref of fieldnames that will not
 
 sub apply {
     my $self = shift;
+    my $id = id $self;
     my $pkref = ( defined $_[0] ? $_[0] : undef );
     my $row;
     my $done = 1;    # by default, changes are done
                      #we are adding a new record if $pos < 0
-    $self->{log}->debug( "apply: pos : " . $self->{pos} );
-    if ( $self->{pos} < 0 ) {
-        $self->{log}->debug("New row");
+    $log{ $id }->debug( "apply: pos : " . $states{ $id }->{pos} );
+    if ( $states{ $id }->{pos} < 0 ) {
+        $log{ $id }->debug("New row");
     }
 
     # deleting a (or some) record
-    for my $p ( @{ $self->{pos2del} } ) {
-        $self->{dman}->set_row_pos($p);
-        $self->{dman}->delete;
+    for my $p ( @{ $widgets{ $id }->{pos2del} } ) {
+        $dman{ $id }->set_row_pos($p);
+        $dman{ $id }->delete;
     }
 
     #$self->_dman_update_rows($self->{pos2del}) unless ($arg{form_only});
 
-    $self->{log}->debug( "items in pos2del: " . scalar @{ $self->{pos2del} } );
-    if ( scalar @{ $self->{pos2del} } ) {
-        $self->{pos2del} = [];
-        $self->{changed} = 0;
-        my $last = $self->{dman}->row_count;
+    $log{ $id }->debug( "items in pos2del: " . scalar @{ $widgets{ $id }->{pos2del} } );
+    if ( scalar @{ $widgets{$id}->{pos2del} } ) {
+        $widgets{ $id }->{pos2del} = [];
+        $states{ $id }->{changed} = 0;
+        my $last = $dman{ $id }->row_count;
 
         # $self->set_record_status_label;
         if ( $last > 0 ) {
@@ -344,42 +498,42 @@ sub apply {
 
     # return value: number of fields updated - don't save if there are none
     my $count = $self->_update_fields($pkref);
-    $self->{log}->debug( "count updated ", $count );
+    $log{ $id }->debug( "count updated ", $count );
     if ($count) {
-        $self->{log}->debug("dman->save");
-        $done = $self->{dman}->save;
+        $log{ $id }->debug("dman->save");
+        $done = $dman{ $id }->save;
     }
 
     #fetch the value of the autoinc pk to pass them on after insert.
     my %pk_val;
-    my @pk = $self->{dman}->get_autoinc_primarykeys;
+    my @pk = $dman{ $id }->get_autoinc_primarykeys;
 
     for my $pk (@pk) {
-        $self->{log}->debug( "Primary Key: " . $pk );
-        my $value = $self->{dman}->get_field($pk);
+        $log{ $id }->debug( "Primary Key: " . $pk );
+        my $value = $dman{ $id }->get_field($pk);
         $pk_val{$pk} = $value;
     }
 
     #push @pk_val, $id
 
-    if ( $done && $self->{after_insert} ) {
-        my $coderef = $self->{after_insert};
+    if ( $done && $events{ $id }->{after_insert} ) {
+        my $coderef = $events{ $id }->{after_insert};
         &$coderef( undef, \%pk_val );
     }
 
     #if ($done && $self->{pos}<0){
-    if ( $done && $self->{inserting} ) {
-        my $last = $self->{dman}->row_count - 1;
+    if ( $done && $states{ $id}->{inserting} ) {
+        my $last = $dman{ $id }->row_count - 1;
         $last = ( $last < 0 ? 0 : $last );
-        $self->{log}->debug( "last is " . $last );
+        $log{ $id }->debug( "last is " . $last );
         $self->_display_data($last);
-        $self->{inserting} = 0;
+        $states{ $id }->{inserting} = 0;
 
         # $self->{rec_spinner_callback}->(); fait dans diplay_data
 
 =for comment
 		 if ($self->{rec_spinner}){
-		  #	my $last = $self->{dman}->row_count;
+		  #	my $last = $dman{ $id }->row_count;
 			$self->{rec_spinner}->signal_handler_block( $self->{rs_value_changed_signal} );
         		$self->{rec_spinner}->set_value($last+1);
 	        	$self->{rec_spinner}->signal_handler_unblock( $self->{rs_value_changed_signal} );
@@ -388,7 +542,7 @@ sub apply {
 
     }
     if ($done) {
-        $self->{changed} = 0;
+        $states{ $id }->{changed} = 0;
         $self->_save_subforms;
 
         $self->_set_record_status_label;
@@ -398,62 +552,63 @@ sub apply {
 
 sub _update_fields {
     my $self = shift;
+    my $id = id $self;
     my @pk =
-      ( defined $_[0] ? @{ $_[0] } : $self->{dman}->get_autoinc_primarykeys );
+      ( defined $_[0] ? @{ $_[0] } : $dman{ $id }->get_autoinc_primarykeys );
 
 #updating a new or an existing record
 #foreach widget in the form, get the value from the widget and place it in the field unless it's a primary key
 #with an autogenerated value
-
+    $log{$id}->debug("_update_fields \@pk : ", join(" ", @pk), " col2save: ",  join(" ", @{ $widgets{ $id }->{col2save} }));
     my $count_updated = 0;
-    foreach my $id ( @{ $self->{col2save} } ) {
+    foreach my $idcol ( @{ $widgets{ $id }->{col2save} } ) {
 
 # {datawidgets}: href to $widget object - different from the datawidgets param in the constructor
-        $self->{log}->debug( "updating field ", $id );
-        if ( exists $self->{datawidgets}->{$id} ) {
+        $log{ $id }->debug( "updating field ", $idcol );
+        if ( exists $widgets{ $id }->{datawidgets}->{$idcol} ) {
 
-            #@pk = $self->{dman}->get_autoinc_primarykeys;
+            #@pk = $dman{ $id }->get_autoinc_primarykeys;
 
             #if ($id ~~ @pk)  {
-            if ( grep /^$id$/, @pk ) {
-                $self->{log}->debug( $id, " not done because it's a pk" );
+            if ( grep /^$idcol$/, @pk ) {
+                $log{ $id }->debug( $idcol, " not done because it's a pk" );
             } else {
-                my $w = $self->{datawidgets}->{$id};
-                $self->{log}->debug( $self->{datawidgetsName}->{$id} );
-                my %getter  = $self->{childclass}->_get_getter;
-                my $coderef = $getter{ $self->{datawidgetsName}->{$id} };
-
-                #$id only used by Wxform
-                my $v = &$coderef( $self, $w, $id );
-                $self->{log}->debug(
-                    "_update_fields id: $id value: " . ( $v ? $v : "" ) );
+                my $w = $widgets{ $id }->{datawidgets}->{$idcol};
+                $log{ $id }->debug( $widgets{ $id }->{datawidgetsName}->{$idcol} );
+                my %getter  = $child_class{ $id }->_get_getter;
+                my $coderef = $getter{ $widgets{ $id }->{datawidgetsName}->{$idcol} };
+                $log{ $id }->debug(Dumper $coderef);
+                #$idcol only used by Wxform
+                my $v = &$coderef( $self, $w, $idcol );
+                $log{ $id }->debug(
+                    "_update_fields id: $idcol value: " . ( $v ? $v : "" ) );
                 $v = ( $v eq "" ? undef : $v );
-                $self->{log}->debug( $id . ": value undef" )
+                $log{ $id }->debug( $idcol . ": value undef" )
                   unless ( defined $v );
 
                 # if ( defined $v && ( $id ~~ @{$self->{dates_formatted}})){
-                if ( defined $v && defined $self->{hdates_formatted}->{$id} ) {
+                if ( defined $v && defined $widgets{ $id }->{hdates_formatted}->{$idcol} ) {
 
                     #my $ff = $self->{dates_formatters_f}->{$id};
 
                     #my $date = $ff->parse_datetime($v);
-                    $v = $self->_format_date( 1, $id, $v );
+                    $v = $self->_format_date( 1, $idcol, $v );
 
              # $v = $self->{dates_formatters_db}->{$id}->format_datetime($date);
              #$v = $self->dateformatter('%Y-%m-%d', $date);
                 }
 
-                if ( $self->{pos} < -1 ) {
-                    $self->{log}->debug(
-                        "current row pos: " . $self->{dman}->get_row_pos );
+                if ( $states{$id }->{pos} < -1 ) {
+                    $log{ $id }->debug(
+                        "current row pos: " . $dman{ $id }->get_row_pos );
                 }
                 $count_updated++;
-                $self->{dman}->set_field( $id, $v );
-                $self->{log}->debug("done");
+                $dman{ $id }->set_field( $idcol, $v );
+                $log{ $id }->debug("done");
             }    # not in @pk
         }    # if exists
         else {
-            $self->{log}->debug( $id . " not in data" );
+            $log{ $id }->debug( $idcol . " not in data" );
         }
     }    #foreach
 
@@ -478,32 +633,37 @@ sub _update_fields {
 
 sub next {
     my $self = shift;
-    if ( $self->{auto_apply} && $self->has_changed ) { $self->apply; }
-    $self->_display_data( $self->{dman}->next );
+    my $id = id $self;
+    if ( $states{ $id }->{auto_apply} && $self->has_changed ) { $self->apply; }
+    $self->_display_data( $dman{ $id }->next );
 }
 
 sub previous {
     my $self = shift;
-    if ( $self->{auto_apply} && $self->has_changed ) { $self->apply; }
-    $self->_display_data( $self->{dman}->previous );
+    my $id = id $self;
+    if ( $states{ $id }->{auto_apply} && $self->has_changed ) { $self->apply; }
+    $self->_display_data( $dman{ $id }->previous );
 }
 
 sub first {
     my $self = shift;
-    if ( $self->{auto_apply} && $self->has_changed ) { $self->apply; }
-    $self->_display_data( $self->{dman}->first );
+    my $id = id $self;
+    if ( $states{ $id}->{auto_apply} && $self->has_changed ) { $self->apply; }
+    $self->_display_data( $dman{ $id }->first );
 }
 
 sub last {
     my $self = shift;
-    if ( $self->{auto_apply} && $self->has_changed ) { $self->apply; }
-    $self->_display_data( $self->{dman}->last );
+    my $id = id $self;
+    if ( $states{ $id }->{auto_apply} && $self->has_changed ) { $self->apply; }
+    $self->_display_data( $dman{ $id }->last );
 }
 
 sub _save_subforms {
     my ($self) = @_;
-    return unless ( $self->{auto_apply} );
-    foreach my $sf ( @{ $self->{subform} } ) {
+    my $id = id $self;
+    return unless ( $states{$id }->{auto_apply} );
+    foreach my $sf ( @{ $widgets{ $id }->{subform} } ) {
         $sf->apply if ( $sf->has_changed );
     }
 
@@ -511,13 +671,14 @@ sub _save_subforms {
 
 sub set_widget_value {
     my ( $self, $wid, $x ) = @_;
-    $self->{log}->debug(
+    my $id = id $self;
+    $log{ $id }->debug(
         "set_widget_value: " . $wid . " to " . ( defined $x ? $x : "null" ) );
-    my $w = $self->{builder}->get_object($wid);
+    my $w = $widgets{ $id }->{builder}->get_object($wid);
     if ($w) {
 
-        my %setter  = $self->{childclass}->_get_setter;
-        my $coderef = $setter{ $self->{datawidgetsName}->{$wid} };
+        my %setter  = $child_class{ $id }->_get_setter;
+        my $coderef = $setter{ $widgets{ $id }->{datawidgetsName}->{$wid} };
 
         #$wid used only by wxform
         &$coderef( $self, $w, $x, $wid );
@@ -527,16 +688,17 @@ sub set_widget_value {
 
 sub get_widget_value {
     my ( $self, $wid ) = @_;
+    my $id = id $self;
     my $x;
-    $self->{log}->debug( "get_widget_value: " . $wid );
-    my $w = $self->{builder}->get_object($wid);
-    $self->{log}->debug("no widget found") unless ($w);
-    if ( $w && $self->{datawidgetsName} ) {
-        my %getter  = $self->{childclass}->_get_getter;
-        my $coderef = $getter{ $self->{datawidgetsName}->{$wid} };
+    $log{ $id }->debug( "get_widget_value: " . $wid );
+    my $w = $widgets{ $id }->{builder}->get_object($wid);
+    $log{ $id }->debug("no widget found") unless ($w);
+    if ( $w && $widgets{ $id }->{datawidgetsName} ) {
+        my %getter  = $child_class{ $id }->_get_getter;
+        my $coderef = $getter{ $widgets{ $id }->{datawidgetsName}->{$wid} };
         $x = &$coderef( $self, $w, $wid );
     }
-    $self->{log}->debug( "found: " . ( $x ? $x : " undef" ) );
+    $log{ $id }->debug( "found: " . ( $x ? $x : " undef" ) );
     return ( $x ? $x : "" );
 }
 
@@ -548,11 +710,12 @@ Reflect in the user interface the changes made after the data manager has been q
 
 sub update {
     my ($self) = @_;
-    my @col = $self->{dman}->get_field_names;
-    $self->{log}->debug(
+    my $id = id $self;
+    my @col = $dman{ $id }->get_field_names;
+    $log{ $id }->debug(
         "update cols are " . ( @col ? join( " ", @col ) : " cols undef " ) );
     my $pos;
-    if ( $self->{dman}->row_count > 0 ) {
+    if ( $dman{ $id }->row_count > 0 ) {
 
         #$self->{rec_spinner}->set_value(1) if ($self->{rec_spinner});
         $pos = 0;
@@ -562,8 +725,8 @@ sub update {
     }
     $self->_display_data($pos);
     my $first = ( $pos < 0 ? 0 : 1 );
-    $self->_set_rs_range( $first, $self->{dman}->row_count );
-    my $coderef = $self->{rec_spinner_callback};
+    $self->_set_rs_range( $first, $dman{ $id }->row_count );
+    my $coderef = $events{ $id }->{rec_spinner_callback};
     &$coderef($self);
 }
 
@@ -573,17 +736,18 @@ sub update {
 # $id is the field id
 # $v the date string from the form (if in_db is 1) or from the db (if in_db is 0)
 sub _format_date {
-    my ( $self, $in_db, $id, $v ) = @_;
-    $self->{log}->debug( "format_date received date: " . $v );
+    my ( $self, $in_db, $idcol, $v ) = @_;
+    my $id = id $self;
+    $log{ $id }->debug( "format_date received date: " . $v );
     my ( $pos1, $pos2 ) = ( $in_db ? ( 1, 0 ) : ( 0, 1 ) );
-    my $format = $self->{date_formatters}->{$id}->[$pos1];
+    my $format = $widgets{ $id }->{date_formatters}->{$idcol}->[$pos1];
     my $f      = $self->_get_dateformatter($format);
     my $dt     = $f->parse_datetime($v) or croak( $f->errmsg );
-    $self->{log}->debug( "format_date:  date time object ymd: " . $dt->ymd );
-    $format = $self->{date_formatters}->{$id}->[$pos2];
+    $log{ $id }->debug( "format_date:  date time object ymd: " . $dt->ymd );
+    $format = $widgets{ $id }->{date_formatters}->{$idcol}->[$pos2];
     $f      = $self->_get_dateformatter($format);
     my $r = $f->format_datetime($dt) or croak( $f->errmsg );
-    $self->{log}->debug( "format_date formatted date: " . $r );
+    $log{ $id }->debug( "format_date formatted date: " . $r );
 
     return $r;
 
@@ -593,25 +757,26 @@ sub _format_date {
 # return an existing formatter
 sub _get_dateformatter {
     my ( $self, $format ) = @_;
-    my %hf = %{ $self->{dates_formatters} };
+    my $id = id $self;
+    my %hf = %{ $widgets{ $id }->{dates_formatters} };
     my $f;
     if ( exists $hf{$format} ) {
-        $self->{log}->debug(
+        $log{ $id }->debug(
             "get_dateformatter : return an existing formatter for " . $format );
         $f = $hf{$format};
     } else {
-        $self->{log}
+        $log{ $id }
           ->debug( "get_dateformatter: new formatter for " . $format );
         $f = new DateTime::Format::Strptime(
             pattern   => $format,
-            locale    => $self->{locale},
-            time_zone => $self->{time_zone},
+            locale    => $widgets{ $id }->{locale},
+            time_zone => $widgets{$id }->{time_zone},
             on_error  => 'undef',
         );
         $hf{$format} = $f;
 
     }
-    $self->{dates_formatters} = \%hf;
+    $widgets{ $id }->{dates_formatters} = \%hf;
     return $f;
 }
 
@@ -622,7 +787,7 @@ Returns the data manager to be queried
 =cut
 
 sub get_data_manager {
-    return shift->{dman};
+    return $dman{ id shift};
 }
 
 1;
