@@ -180,8 +180,12 @@ sub set_field {
 
             $row = $self->{current_row};
         }
-
-        $row->set_column( $id, $value )
+        try {
+         $row->set_column( $id, $value )
+     } catch {
+         $self->{log}->error( "No column named " . $id . " in row" );
+        
+     }
           ; #or die(__PACKAGE__ . " no method found to set value " . $value . " in the column " . $id . " entries are ".  join(" ", keys %{ $self->{fieldSetter} }));
     }
 }
