@@ -90,7 +90,7 @@ sub new {
             $sth->execute;
         }
         catch {
-            die $self->{dbh}->errstr;
+            croak $self->{dbh}->errstr;
         };
     }
 
@@ -250,7 +250,7 @@ sub query {
 
         }
         catch {
-            die $self->{dbh}->errstr . " " . $local_sql;
+            croak $self->{dbh}->errstr . " " . $local_sql;
         };
 
 #die $local_sql;
@@ -444,7 +444,7 @@ sub save {
         my $v = $self->get_field($fieldname);
         $self->{log}->debug( "push on bind_values "
               . $fieldname . " : "
-              . ( defined $v ? $v : " unded" ) );
+              . "'" . ( defined $v ? $v : " undef" ) . "'" );
         push @bind_values, $self->get_field($fieldname);
 
         #}
@@ -716,7 +716,7 @@ C<sql> a hash ref with the following keys:
 
 =item *
 
-C<select> or C<select_distinct> : a comma delimited string of the field names, default to * if left unspecified.
+C<select> or C<select_distinct> : a comma delimited string of the field names, default to * if left unspecified. Field aliasing is supported.
 
 =item *
 
