@@ -35,7 +35,7 @@ sub new {
     };
     $self->{cols}  = [];
     $self->{hcols} = {};
-    $self->{log}   = Log::Log4perl->get_logger(__PACKAGE__);
+    $self->{rs_log}   = Log::Log4perl->get_logger(__PACKAGE__); #log is used in the derived class
     bless $self, $class;
 }
 
@@ -62,7 +62,7 @@ sub rs_init {
 sub rs_move {
     my ( $self, $offset, $absolute ) = @_;
 
-#$self->{log}->debug("rs_move : offset ", (defined $offset ? $offset : "undef"), " abs: ", (defined $absolute ? $absolute : "undef") );
+#$self->{rs_log}->debug("rs_move : offset ", (defined $offset ? $offset : "undef"), " abs: ", (defined $absolute ? $absolute : "undef") );
 
     my ( $new_batch_group, $new_position );
 
@@ -87,7 +87,7 @@ sub rs_move {
         # Check if we need to roll to another slice of our recordset
         $new_batch_group = int( $new_position / $self->{batch_size} );
 
-        #$self->{log}->debug("new_batch_group: ", $new_batch_group);
+        #$self->{rs_log}->debug("new_batch_group: ", $new_batch_group);
         if ( defined $self->{batch_pos} ) {
             if ( $self->{batch_group} != $new_batch_group ) {
                 $self->{batch_group} = $new_batch_group;
@@ -97,7 +97,7 @@ sub rs_move {
             }    #else {
 
 #same batch
-#	$self->{log}->debug("same batch - keeping the same ", $self->{batch_size}, " values ");
+#	$self->{rs_log}->debug("same batch - keeping the same ", $self->{batch_size}, " values ");
 #}
 
         } else {
@@ -116,7 +116,7 @@ sub rs_move {
 
     }
 
-    #$self->{log}->debug("batch_pos: ", $self->{batch_pos});
+    #$self->{rs_log}->debug("batch_pos: ", $self->{batch_pos});
     #return @data;
     return $isnewbatch;
 
@@ -140,9 +140,9 @@ sub _fetch_new_batch {
 
     my $batch_count = $self->row_count;
 
- #$self->{log}->debug("_fetch_new_batch batch_group : ". $self->{batch_group} );
+ #$self->{rs_log}->debug("_fetch_new_batch batch_group : ". $self->{batch_group} );
 
-#$self->{log}->debug("_fetch_new_batch lower: " . $lower . "  count : " . $batch_count . " upper ". $upper);
+#$self->{rs_log}->debug("_fetch_new_batch lower: " . $lower . "  count : " . $batch_count . " upper ". $upper);
 
     if ( ( $batch_count == 0 ) || ( $batch_count == $lower ) ) {
 
@@ -154,7 +154,7 @@ sub _fetch_new_batch {
 
         # First, we have to delete anything in $self->{records}
         $self->{records} = ();
-        $self->{log}->debug("_fetch_new_batch called on an empty recordset");
+        $self->{rs_log}->debug("_fetch_new_batch called on an empty recordset");
         $isnewbatch = 0;
     } else {
 
@@ -175,7 +175,7 @@ sub _fetch_new_batch {
          # $local_sql .= " ( " . join( ",", $self->{keyset}[$counter] ) . " ),";
                 my @pk_vals = @{ $all_pk_vals[$counter] };
 
-#$self->{log}->debug("array for row ", $counter, " is ", Dumper(@pk_vals), " value for ", $pk, " is ", $pk_vals[$pk_order]);
+#$self->{rs_log}->debug("array for row ", $counter, " is ", Dumper(@pk_vals), " value for ", $pk, " is ", $pk_vals[$pk_order]);
 #push @vals_for_pk, $pk_vals[$pk_order] unless ($seen{ $pk_vals[$pk_order] }++);
                 if ( $self->{pkvalues_filter} ) {
 
@@ -230,7 +230,7 @@ sub rs_set_rows {
     #die ( ref $data);
     $self->{records} = $data;
 
-    #$self->{log}->debug("rs_set_rows : ", Dumper($self->{records}));
+    #$self->{rs_log}->debug("rs_set_rows : ", Dumper($self->{records}));
 }
 
 #Param: $data hash ref for one row
@@ -240,7 +240,7 @@ sub rs_add_row {
     #push @{$self->{records}}, $data;
     $self->{records}[ $self->{batch_pos} ] = $data;
 
-    #$self->{log}->debug("rs_add_row : ", Dumper($self->{records}));
+    #$self->{rs_log}->debug("rs_add_row : ", Dumper($self->{records}));
 
 }
 
@@ -254,16 +254,15 @@ sub set_field {
     my ( $self, $fieldname, $value ) = @_;
     $self->{records}[ $self->{batch_pos} ]->{$fieldname} = $value;
 
-    #$self->{log}->debug("set_field : ", Dumper ($self->{records}));
+    #$self->{rs_log}->debug("set_field : ", Dumper ($self->{records}));
 
 }
 
 sub get_field {
     my ( $self, $fieldname ) = @_;
 
-# $self->{log}->debug("get_field - name : " . $fieldname . "\nRecords: " . (Dumper $self->{records}));
-    carp(
-"get_field($fieldname) called on an empty recordset ! Did you forget to call set_row_pos(0) after query ?"
+# $self->{rs_log}->debug("get_field - name : " . $fieldname . "\nRecords: " . (Dumper $self->{records}));
+    carp("get_field($fieldname) called on an empty recordset ! Did you forget to call set_row_pos(0) after query ?"
     ) unless ( $self->{records} );
     my $data = $self->{records}[ $self->{batch_pos} ]->{$fieldname};
     return $data;
@@ -297,7 +296,7 @@ sub row_count {
         $count = 0;
     }
 
-   # $self->{log}->debug("row_count :  counting ", $count_this , " : ", $count);
+   # $self->{rs_log}->debug("row_count :  counting ", $count_this , " : ", $count);
     return $count;
 }
 
@@ -366,7 +365,7 @@ sub get_primarykeys {
 sub set_row_pos {
     my ( $self, $pos, $isrelative ) = @_;
 
-    #$self->{log}->debug("set_row_pos: " . $pos  );
+    #$self->{rs_log}->debug("set_row_pos: " . $pos  );
     #$self->_move(undef, $pos);
     my @batch_pk_vals;
     my $newbatch;
@@ -380,7 +379,7 @@ sub set_row_pos {
     if ($newbatch) {
 
         #die $pos;
-        #$self->{log}->debug("set inserting to 0");
+        #$self->{rs_log}->debug("set inserting to 0");
         $self->{inserting} = 0;
         my $data =
           $self->_get_rows_from_batch( $self->get_pkvalues_from_batch );
@@ -399,11 +398,11 @@ sub get_field_type {
         $type = ( $fieldtype{$type} ? $fieldtype{$type} : $type );
     } else {
 
-        #$self->{log}->debug (Dumper $self->{column_info});
+        #$self->{rs_log}->debug (Dumper $self->{column_info});
         $type = "varchar";
     }
 
-    #$self->{log}->debug("get_field_type for ".   $fieldname . " : " . $type);
+    #$self->{rs_log}->debug("get_field_type for ".   $fieldname . " : " . $type);
     return $type;
 }
 
@@ -411,7 +410,7 @@ sub new_row {
 
     my $self = shift;
 
-    #$self->{log}->debug("new row called : set inserting flag");
+    #$self->{rs_log}->debug("new row called : set inserting flag");
     my $newposition =
       $self->row_count;    # No need to add one, as the array starts at zero.
 
@@ -423,7 +422,7 @@ sub new_row {
     #$self->{records}[$self->{slice_position}] = $self->_assemble_new_record;
     $self->rs_add_row( $self->_assemble_new_record );
 
-    # $self->{log}->debug("new row rows are : ", Dumper($self->rs_get_rows));
+    # $self->{rs_log}->debug("new row rows are : ", Dumper($self->rs_get_rows));
 }
 
 sub _assemble_new_record {
@@ -432,7 +431,7 @@ sub _assemble_new_record {
 
     my $self = shift;
 
-    #$self->{log}->debug("_assemble_new_record");
+    #$self->{rs_log}->debug("_assemble_new_record");
     my $new_record;
 
     # First, we create fields with default values from the database ...
@@ -443,7 +442,7 @@ sub _assemble_new_record {
         if ( $default && $self->{server} =~ /microsoft/i ) {
             $default = $self->parse_sql_server_default($default);
         }
-        $self->{log}->debug(
+        $self->{rs_log}->debug(
             "set database's default value ",
             $self->{defaults}->{$fieldname},
             " for field ", $fieldname
@@ -453,7 +452,7 @@ sub _assemble_new_record {
 
     # ... and then we set user-defined defaults
     foreach my $fieldname ( keys %{ $self->{defaults} } ) {
-        $self->{log}->debug(
+        $self->{rs_log}->debug(
             "set user's default value ",
             $self->{defaults}->{$fieldname},
             " for field ", $fieldname
@@ -496,7 +495,7 @@ sub _last_insert_id {
         }
 
     }
-    $self->{log}->debug( "_last_instert_id : ",
+    $self->{rs_log}->debug( "_last_instert_id : ",
         ( defined $primary_key ? $primary_key : " undef" ) );
     return $primary_key;
 
@@ -510,14 +509,14 @@ sub _init_pks {
     # DBIDataManager and SqlADataManger : table name is not in the same field
     my $table = $self->{sql}->{from};
     $table = ( defined $table ? $table : $self->{table} );
-    $self->{log}->debug( "_init_pks: ", $table );
+    $self->{rs_log}->debug( "_init_pks: ", $table );
 
     try {
         $sth = $self->{dbh}->primary_key_info( undef, undef, $table );
 
     }
     catch {
-        $self->{log}->debug( $self->{dbh}->errstr );
+        $self->{rs_log}->debug( $self->{dbh}->errstr );
         return;
     };
 
@@ -528,20 +527,20 @@ sub _init_pks {
 
     while ( my $row = $sth->fetchrow_hashref ) {
 
-        $self->{log}->debug( "Detected primary key : " . $row->{COLUMN_NAME} );
+        $self->{rs_log}->debug( "Detected primary key : " . $row->{COLUMN_NAME} );
         try {
             $sth2 = $self->{dbh}
               ->column_info( undef, undef, $table, $row->{COLUMN_NAME} );
         }
         catch {
-            $self->{log}->debug( $self->{dbh}->errstr );
+            $self->{rs_log}->debug( $self->{dbh}->errstr );
         };
 
         while ( my $ir = $sth2->fetchrow_hashref ) {
             if ( $ir->{mysql_is_auto_increment} ) {
 
                 # Dumper($ir);
-                # $self->{log}->debug("ai pk type: ", $ir->{TYPE});
+                # $self->{rs_log}->debug("ai pk type: ", $ir->{TYPE});
                 $self->{auto_incrementing} = 1;
             }
 
@@ -566,7 +565,7 @@ sub _use_sth_info {
     my %hcols = map { $_ => 1 } @{ $self->{cols} };
     $self->{hcols} = \%hcols;
     my @type = @{ $sth->{'TYPE'} };
-    $self->{log}->debug( "TYPE: ", join( " ", @type ) );
+    $self->{rs_log}->debug( "TYPE: ", join( " ", @type ) );
     my $pos     = 0;    #http://docstore.mik.ua/orelly/linux/dbi/ch06_01.htm
     my %sqltype = (
         1  => 'char',
@@ -592,14 +591,14 @@ sub _use_sth_info {
     for my $name ( @{ $self->{cols} } ) {
         my $type = lc $type[ $pos++ ];
 
-        #$self->{log}->debug("searching " . $type);
+        #$self->{rs_log}->debug("searching " . $type);
         #if ($self->{server}){
         if ( $type =~ /-?\d{1,2}/ )
         { #mysql gives type as given in sqltype above, DBI::CSV (where $self->{server} is undef) gives the type as an array of string
             $type = $sqltype{$type};
         }
 
-        $self->{log}->debug( $name . " type: " . $type );
+        $self->{rs_log}->debug( $name . " type: " . $type );
         $self->{column_info}->{$name}->{TYPE_NAME} = $type;
     }
 }
@@ -607,7 +606,7 @@ sub _use_sth_info {
 sub _add_to_cols {
     my ( $self, $fieldname ) = @_;
     my %hcols = %{ $self->{hcols} };
-    $self->{log}->debug( "_add_to_cols ", $fieldname )
+    $self->{rs_log}->debug( "_add_to_cols ", $fieldname )
       unless ( defined $hcols{$fieldname} );
     push @{ $self->{cols} }, $fieldname unless ( defined $hcols{$fieldname} );
     $hcols{$fieldname}++;
