@@ -240,6 +240,7 @@ sub _get_rows_from_batch {
         : $self->{select_param}->{-where} );
 
 #$self->{log}->debug("original select parameters : \n", Dumper($self->{select_param}));
+#$self->{log}->debug("where_orig ", join(" ", @where_orig));
 #my @in;
 
     #my @all_pk_vals = @{$self->{keyset}};
@@ -266,7 +267,7 @@ sub _get_rows_from_batch {
   #my $value = $pk_val->[]
   #$self->{log}->debug("pk :", $pk, " value: ", $pk_val);
  
-   push @vals_for_pk, $pk_val unless ( $seen{$pk_val}++ );
+            push @vals_for_pk, $pk_val unless ( $seen{$pk_val}++ );
 
             #push @where_new, %where;
 
@@ -285,11 +286,16 @@ sub _get_rows_from_batch {
     my ( $stm, @bind ) = $self->{sql}->select( ( %{ $self->{select_param} } ) );
 
     my $sth = $self->_execute( $stm, @bind );
-
+    return unless ($sth);
     #$self->{records} = $sth->fetchall_arrayref({});
     #$self->rs_set_rows( $sth->fetchall_arrayref({}) );
-    %w = @where_orig;
-    $self->{select_param}->{-where} = \%w;
+   if ( @where_orig > 1){
+        %w = @where_orig;
+        $self->{select_param}->{-where} = \%w;
+    } 
+    else {
+         $self->{select_param}->{-where} = $where_orig[0];
+    }
 
     # return an array ref of hash ref: each hash is {field_name => field_Value}
     my $data = $sth->fetchall_arrayref( {} );
